#include "board_12F1822.h"
#include "data.h"

__EEPROM_DATA(0x55, 0xAA, INVERSION_MASK /* inverted outputs */, 0x01 /* last output no */, 0xFF, 0xFF, 0xFF, 0xFF);

//==============================================================================
TSystemFlags SystemFlags;

//==============================================================================
// private
uint8_t InversionMask;
uint8_t LastOutputNumber;
static void LoadSettings(void);
static void ActivateOutput(void);

//==============================================================================
void Board_Init(){
   //OSCCON = _OSCCON_IRCF3_MASK | _OSCCON_IRCF2_MASK | _OSCCON_IRCF0_MASK; // 500 kHz
   PORTA = 0;
   CM1CON0 = 0;
   ANSELA = 0; // all of pins are digital I/O
   WPUA = INPUT_BUTTON_MASK; // pull-up only at input pin
   OPTION_REGbits.nWPUEN = 0;// global pull-ups enable
   
   LoadSettings();
   ActivateOutput();
   
   TRISA = ~(OUTPUT_1_MASK | OUTPUT_2_MASK | OUTPUT_3_MASK | OUTPUT_4_MASK);  // I/O direction
}

//==============================================================================
// 200Hz/5ms at default 500KHz Fosc
// ratio is 500000/4/200 = 625 = 125 * 5
void Systick_Init(){
    PR2 = 125 - 1;
    TMR2 = 0;
    PIR1bits.TMR2IF = 0;
    T2CON = 
    // prescaler = 1            
    (0<< _T2CON_T2CKPS1_POSN)|(0<< _T2CON_T2CKPS0_POSN)|
    // postscaler = 5
    (0<< _T2CON_T2OUTPS3_POSN)|(1<< _T2CON_T2OUTPS2_POSN)|(0<< _T2CON_T2OUTPS1_POSN)|(0<< _T2CON_T2OUTPS0_POSN)|
    (1<< _T2CON_TMR2ON_POSN);
}

//==============================================================================
static void LoadSettings(){
    InversionMask = (uint8_t)eeprom_read(EE_ADDR_SETTING_INVERSION_MASK);
    LastOutputNumber = (uint8_t)eeprom_read(EE_ADDR_SETTING_OUTPUT_NUMBER);
}

//==============================================================================
void SaveSettings(){
    eeprom_write(EE_ADDR_SETTING_OUTPUT_NUMBER, LastOutputNumber);
}

//==============================================================================
static void ActivateOutput() {
    uint8_t port_value = 0;

    // validate output number
   if (LastOutputNumber == 0 || LastOutputNumber > OUTPUT_COUNT){
       LastOutputNumber = 1;
   }

    switch (LastOutputNumber) {
        case (uint8_t)1:
            port_value = OUTPUT_1_MASK;
            break;
        case (uint8_t)2:
            port_value = OUTPUT_2_MASK;
            break;
        case (uint8_t)3:
            port_value = OUTPUT_3_MASK;
            break;
        case (uint8_t)4:
            port_value = OUTPUT_4_MASK;
            break;

        default:
            break;
    }
    LATA = (uint8_t)(port_value ^ InversionMask);
}

//==============================================================================
void AdvanceOutput(){
    LastOutputNumber++;
    ActivateOutput();
}

//==============================================================================
void Systick_WaitSync(){
    while (!PIR1bits.TMR2IF);
    PIR1bits.TMR2IF = 0;
}

//==============================================================================
void Delay(uint8_t ms5){
    while (ms5--)
        Systick_WaitSync();
}

//==============================================================================
inline void EnableWakeUpSource(void){
    IOCAF &= (uint8_t)(IOCAF ^ 0xFF); // reset wakeup conditions
    IOCAN |= INPUT_BUTTON_MASK;
    INTCONbits.IOCIE = 1;             // enable IOC interrupt
}

//==============================================================================
inline void DisableWakeUpSource(void){
    INTCONbits.IOCIE = 0;             // disable IOC interrupt
    IOCAN &= ~INPUT_BUTTON_MASK;
    IOCAF &= (uint8_t)(IOCAF ^ 0xFF); // reset wakeup conditions
}

//==============================================================================
void SleepMode(){
    EnableWakeUpSource();
    SLEEP();
    DisableWakeUpSource();
}

