#include "board_12F675.h"
#include "data.h"

__EEPROM_DATA(0x55, 0xAA, INVERSION_MASK /* inverted outputs */, 0x01 /* last output no */, 0xFF, 0xFF, 0xFF, 0xFF);

//==============================================================================
TSystemFlags SystemFlags;

//==============================================================================
// private
uint8_t InversionMask;
uint8_t LastOutputNumber;
static void LoadSettings(void);
static void ActivateOutput(void);

//==============================================================================
void Board_Init(){
   GPIO = 0;
   ANSEL = 0;
   CMCON = _CMCON_CM2_MASK | _CMCON_CM1_MASK | _CMCON_CM0_MASK;
   WPU = INPUT_BUTTON_MASK; // pull-up only at input pin
   OPTION_REGbits.nGPPU = 0; // pull-ups
   
   LoadSettings();
   ActivateOutput();
   
   TRISIO = ~(OUTPUT_1_MASK | OUTPUT_2_MASK | OUTPUT_3_MASK | OUTPUT_4_MASK);  // I/O direction
}

//==============================================================================
// 200Hz/5ms at default 500KHz Fosc
// ratio is 500000/4/200 = 625 = 125 * 5
void Systick_Init(){
    TMR1 = -5000;
    PIR1bits.TMR1IF = 0;
    // prescaler = 1            
    T1CONbits.TMR1ON = 1;
}

//==============================================================================
static void LoadSettings(){
    InversionMask = (uint8_t)eeprom_read(EE_ADDR_SETTING_INVERSION_MASK);
    LastOutputNumber = (uint8_t)eeprom_read(EE_ADDR_SETTING_OUTPUT_NUMBER);
}

//==============================================================================
void SaveSettings(){ 
    eeprom_write(EE_ADDR_SETTING_OUTPUT_NUMBER, LastOutputNumber);
}

//==============================================================================
static void ActivateOutput() {
    uint8_t port_value = 0;

    // validate output number
   if (LastOutputNumber == 0 || LastOutputNumber > OUTPUT_COUNT){
       LastOutputNumber = 1;
   }

    switch (LastOutputNumber) {
        case (uint8_t)1:
            port_value = OUTPUT_1_MASK;
            break;
        case (uint8_t)2:
            port_value = OUTPUT_2_MASK;
            break;
        case (uint8_t)3:
            port_value = OUTPUT_3_MASK;
            break;
        case (uint8_t)4:
            port_value = OUTPUT_4_MASK;
            break;

        default:
            break;
    }
    GPIO = (uint8_t)(port_value ^ InversionMask);
}

//==============================================================================
void AdvanceOutput(){
    LastOutputNumber++;
    ActivateOutput();
}

//==============================================================================
void Systick_WaitSync(){
    while (!PIR1bits.TMR1IF);
    TMR1 = -5000;
    PIR1bits.TMR1IF = 0;
}

//==============================================================================
inline void EnableWakeUpSource(void){
    IOC |= INPUT_BUTTON_MASK;
    GPIO;                           // reset interrupt condition
    INTCONbits.GPIE = 1;             // enable IOC interrupt
}

//==============================================================================
inline void DisableWakeUpSource(void){
    INTCONbits.GPIE = 0;             // disable IOC interrupt
    IOC &= ~INPUT_BUTTON_MASK;
}

//==============================================================================
void SleepMode(){
    EnableWakeUpSource();
    SLEEP();
    DisableWakeUpSource();
}

