#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include "board_ATtiny13A.h"
#include "data.h"

const uint8_t EE_INITIAL_SETTINGS[] EEMEM = {0x55, 0xAA, INVERSION_MASK /* inverted outputs */, 0x01 /* last output no */, 0xFF, 0xFF, 0xFF, 0xFF}  ;

//==============================================================================
TSystemFlags SystemFlags;

// private
uint8_t InversionMask;
uint8_t LastOutputNumber;
static void LoadSettings(void);
static void ActivateOutput(void);

//==============================================================================
void Board_Init(){
   PRR |= PRADC; // ADC power off
   LoadSettings();
   ActivateOutput();
   DDRB = ~INPUT_BUTTON_MASK; // I/O direction
}

//==============================================================================
// 200Hz/5ms at 600KHz (4.8MHz/8)
// ratio is 600000/200 = 3000 = 64 * 47
void Systick_Init(){
	TCNT0 = 0;
	OCR0A = 47 - 1;
	TIFR0 |= (1<< OCF0A);
    // prescaler = 64, CTC mode (WGM[2..0] = 010, SC[2..0] =  011)
	TCCR0A = (1<< WGM01)|(0<< WGM00);
	TCCR0B = (0<< WGM02)| (0<< CS02)|(1<< CS01)|(1<< CS00);
	
}

//==============================================================================
static void LoadSettings(){
    InversionMask = eeprom_read_byte((const uint8_t *)EE_ADDR_SETTING_INVERSION_MASK);
    LastOutputNumber = eeprom_read_byte((const uint8_t *)EE_ADDR_SETTING_OUTPUT_NUMBER);
}

//==============================================================================
void SaveSettings(){
    eeprom_write_byte((uint8_t *)EE_ADDR_SETTING_OUTPUT_NUMBER, LastOutputNumber);
}

//==============================================================================
static void ActivateOutput() {
    uint8_t port_value = 0;

    // validate output number
    if (LastOutputNumber == 0 || LastOutputNumber > OUTPUT_COUNT){
	    LastOutputNumber = 1;
    }

    switch (LastOutputNumber) {
        case (uint8_t)1:
            port_value = OUTPUT_1_MASK;
            break;
        case (uint8_t)2:
            port_value = OUTPUT_2_MASK;
            break;
        case (uint8_t)3:
            port_value = OUTPUT_3_MASK;
            break;
        case (uint8_t)4:
            port_value = OUTPUT_4_MASK;
            break;

        default:
            break;
    }
	port_value = (port_value ^ InversionMask);
	port_value |= INPUT_BUTTON_MASK ; // preserve pull-up on button line
    PORTB = port_value;
}

//==============================================================================
void AdvanceOutput(){
    LastOutputNumber++;
    ActivateOutput();
}

//==============================================================================
void Systick_WaitSync(){
	while (!(TIFR0 & (1<< OCF0A)));
	TIFR0 |= (1<< OCF0A);
}

//==============================================================================
void Delay(uint8_t ms5){
    while (ms5--)
        Systick_WaitSync();
}

//==============================================================================
inline void EnableWakeUpSource(void){
	PCMSK |= INPUT_BUTTON_MASK;  // enable interrupt on the pin change
	GIFR |= (1<< PCIF);			 // reset interrupt flag
	GIMSK |= (1<< PCIE);		 // enable interrupt
	sei();						 // enable interrupts globally
}

//==============================================================================
inline void DisableWakeUpSource(void){
	PCMSK &= ~INPUT_BUTTON_MASK; // disable interrupt from PCINT
	cli();						 // disable interrupts globally
}

//==============================================================================
void SleepMode(){
	EnableWakeUpSource();
	MCUCR |= (1<< SE)|(1<< SM1)|(0<< SM0); // enable power down
	
	sleep_cpu();
	
	MCUCR &= ~(1<< SE); // disable sleep mode
	DisableWakeUpSource();
}


//==============================================================================
ISR(PCINT0_vect){
	GIMSK &= ~(1<< PCIE);		 // disable interrupt from PCINT
	GIFR |= (1<< PCIF);			 // reset from PCINT interrupt flag 
}
