#include "data.h"
#include "mcu.h"
#include "user_input.h"

//==============================================================================
#define DEBOUNCE_RELOAD_VALUE       (uint8_t)10 // unit is 5ms

//==============================================================================
static uint8_t DebounceCounter;
static uint8_t PreviousKeyState;

//==============================================================================
void KeyScan_Init(){
    PreviousKeyState = 0xAA;
    SystemFlags.KeyPressed = 0;
    DebounceCounter = DEBOUNCE_RELOAD_VALUE;
}

//==============================================================================
// should be called every system tick
void KeyScan_Process(){
    uint8_t keyState = (BUTTON_LINE_STATE); // line is 0 = key pressed
    uint8_t hasChanges = (uint8_t)(PreviousKeyState != keyState);
    PreviousKeyState = keyState;

    if (hasChanges){
        // Reinitialize debounce mechanism
        DebounceCounter = DEBOUNCE_RELOAD_VALUE;
    } else {
        if (DebounceCounter){ // Debounce mechanism is still active
            if (--DebounceCounter == 0){
                // Debounce mechanism confirms new key state 
                SystemFlags.KeyPressed = (uint8_t)(keyState == 0? 1 : 0);
            }
        }
    }
}

//==============================================================================
// returns 1 if key is pressed, 0 otherwise
// resets flag if key is pressed
uint8_t KeyScan_IsKeyPressed(){
    if (SystemFlags.KeyPressed){
        SystemFlags.KeyPressed = 0;
        return 1;
    }
    return 0;
}

